@@ -5,15 +5,13 @@ mod config;
 mod format;
 mod fs;
 
-use config::{Config};
+use config::{Config, ERROR_STATUS_CLI, ERROR_STATUS_FUSE};
 
 use fuser::MountOption;
 
 fn main() {
     let config = Config::from_args();
-    
     let mut options = vec![
-        MountOption::AutoUnmount,
         MountOption::FSName(format!("{}", config.input)),
     ];
     if config.read_only {
@@ -27,7 +25,7 @@ fn main() {
             error!(
                 "No mount point specified; aborting. Use `--mount MOUNT` to specify a mountpoint."
             );
-            std::process::exit(1);
+            std::process::exit(ERROR_STATUS_CLI);
         }
     };
     let cleanup_mount = config.cleanup_mount;
@@ -35,8 +33,16 @@ fn main() {
     let fs = input_format.load(config);
 
     info!("mounting on {:?} with options {:?}", mount, options);
-    fuser::mount2(fs, &mount, &options).unwrap();
-    info!("unmounted");
+    let status = match fuser::mount2(fs, &mount, &options) {
+        Ok(()) => {
+            info!("unmounted");
+            0
+        }
+        Err(e) => {
+            error!("I/O error: {}", e);
+            ERROR_STATUS_FUSE
+        }
+    };
 
     if cleanup_mount {
         if mount.exists() {
@@ -50,4 +56,6 @@ fn main() {
             );
         }
     }
+
+    std::process::exit(status);
 }
